@@ -1,11 +1,13 @@
 from functools import partial
-from typing import Any, Callable, Dict, List, Tuple, Union
+from typing import Any, Callable, Dict, List, Union
 
 from decorator import decorator
+from loguru import logger
 from pyspark.sql import DataFrame, SparkSession
 from pyspark.sql import functions as F
 from pyspark.sql import types as T
 from spacy import Language
+from spacy.tokens import Doc
 
 from edsnlp.pipelines.base import BaseComponent
 
@@ -14,6 +16,8 @@ from .helpers import (
     DataFrames,
     check_spacy_version_for_context,
     get_module,
+    rgetattr,
+    slugify,
 )
 
 
@@ -23,7 +27,7 @@ def pyspark_type_finder(obj):
     """
     try:
         inferred_type = T._infer_type(obj)
-        print(f"Inferred type is {repr(inferred_type)}")
+        logger.info(f"Inferred type is {repr(inferred_type)}")
         return inferred_type
     except TypeError:
         raise TypeError("Cannot infer type for this object.")
@@ -56,7 +60,7 @@ def pipe(
     nlp: Language,
     context: List[str] = [],
     additional_spans: Union[List[str], str] = "discarded",
-    extensions: List[Tuple[str, T.DataType]] = [],
+    extensions: Dict[str, T.DataType] = {},
 ) -> DataFrame:
     """
     Function to apply a spaCy pipe to a pyspark or koalas DataFrame note
@@ -75,10 +79,11 @@ def pipe(
     additional_spans : Union[List[str], str], by default "discarded"
         A name (or list of names) of SpanGroup on which to apply the pipe too:
         SpanGroup are available as `doc.spans[spangroup_name]` and can be generated
-        by some pipes. For instance, the `date` pipe populates doc.spans['dates']
+        by some pipes. For instance, the `eds.dates` pipeline
+        component populates `doc.spans['dates']`
     extensions : List[Tuple[str, T.DataType]], by default []
         Spans extensions to add to the extracted results:
-        FOr instance, if `extensions=["score_name"]`, the extracted result
+        For instance, if `extensions=["score_name"]`, the extracted result
         will include, for each entity, `ent._.score_name`.
 
     Returns
@@ -112,7 +117,7 @@ def pipe(
                     T.StructField("start", T.IntegerType(), False),
                     T.StructField("end", T.IntegerType(), False),
                     *[
-                        T.StructField(extension_name, extension_type, True)
+                        T.StructField(slugify(extension_name), extension_type, True)
                         for extension_name, extension_type in extensions.items()
                     ],
                 ]
@@ -144,7 +149,7 @@ def pipe(
 
             for ent in doc.ents:
                 parsed_extensions = [
-                    getattr(ent._, extension) for extension in extensions.keys()
+                    rgetattr(ent._, extension) for extension in extensions.keys()
                 ]
 
                 ents.append(
@@ -169,7 +174,7 @@ def pipe(
                 for ent in doc.spans.get(spans_name, []):
 
                     parsed_extensions = [
-                        getattr(ent._, extension) for extension in extensions.keys()
+                        rgetattr(ent._, extension) for extension in extensions.keys()
                     ]
 
                     ents.append(
@@ -201,11 +206,107 @@ def pipe(
         extensions=extensions,
     )
 
-    note_nlp = note.withColumn(
+    n_needed_partitions = max(note.count() // 2000, 1)  # Batch sizes of 2000
+
+    note_nlp = note.repartition(n_needed_partitions).withColumn(
         "matches", matcher(F.col("note_text"), *[F.col(c) for c in context])
     )
+
     note_nlp = note_nlp.withColumn("matches", F.explode(note_nlp.matches))
 
     note_nlp = note_nlp.select("note_id", "matches.*")
 
     return note_nlp
+
+
+@module_checker
+def custom_pipe(
+    note: DataFrames,
+    nlp: Language,
+    results_extractor: Callable[[Doc], List[Dict[str, Any]]],
+    dtypes: Dict[str, T.DataType],
+    context: List[str] = [],
+) -> DataFrame:
+    """
+    Function to apply a spaCy pipe to a pyspark or koalas DataFrame note,
+    a generic callback function that converts a spaCy `Doc` object into a
+    list of dictionaries.
+
+    Parameters
+    ----------
+    note : DataFrame
+        A Pyspark or Koalas DataFrame with a `note_id` and `note_text` column
+    nlp : Language
+        A spaCy pipe
+    results_extractor : Callable[[Doc], List[Dict[str, Any]]]
+        Arbitrary function that takes extract serialisable results from the computed
+        spaCy `Doc` object. The output of the function must be a list of dictionaries
+        containing the extracted spans or entities.
+
+        There is no requirement for all entities to provide every dictionary key.
+    dtypes : Dict[str, T.DataType]
+        Dictionary containing all expected keys from the `results_extractor` function,
+        along with their types.
+    context : List[str]
+        A list of column to add to the generated SpaCy document as an extension.
+        For instance, if `context=["note_datetime"], the corresponding value found
+        in the `note_datetime` column will be stored in `doc._.note_datetime`,
+        which can be useful e.g. for the `dates` pipeline.
+
+    Returns
+    -------
+    DataFrame
+        A pyspark DataFrame with one line per extraction
+    """
+
+    if context:
+        check_spacy_version_for_context()
+
+    spark = SparkSession.builder.enableHiveSupport().getOrCreate()
+    sc = spark.sparkContext
+
+    if not nlp.has_pipe("eds.context"):
+        nlp.add_pipe("eds.context", first=True, config=dict(context=context))
+
+    nlp_bc = sc.broadcast(nlp)
+
+    schema = T.ArrayType(
+        T.StructType([T.StructField(key, dtype) for key, dtype in dtypes.items()])
+    )
+
+    @F.udf(schema)
+    def udf(
+        text,
+        *context_values,
+    ):
+
+        if text is None:
+            return []
+
+        nlp_ = nlp_bc.value
+
+        for _, pipe in nlp.pipeline:
+            if isinstance(pipe, BaseComponent):
+                pipe.set_extensions()
+
+        doc = nlp_.make_doc(text)
+        for context_name, context_value in zip(context, context_values):
+            doc._.set(context_name, context_value)
+
+        doc = nlp_(doc)
+
+        results = []
+
+        for res in results_extractor(doc):
+            results.append([res.get(key) for key in dtypes])
+
+        return results
+
+    note_nlp = note.withColumn(
+        "matches", udf(F.col("note_text"), *[F.col(c) for c in context])
+    )
+
+    note_nlp = note_nlp.withColumn("matches", F.explode(note_nlp.matches))
+    note_nlp = note_nlp.select("note_id", "matches.*")
+
+    return note_nlp
